#include "stdafx.h"
#include "ModalDialogCloser.h"
#include "NeMessengerDlg.h"

bool ModalWndInfo::operator == (const ModalWndInfo & other)
{
    bool bSame = false;

    if(type == other.type)
    {
        switch (type)
        {
        case mtMfcDialog:
            bSame = (pDialog == other.pDialog);
            break;
        case mtDuiWnd:
            bSame = (pDuiModalWnd == other.pDuiModalWnd);
            break;
        default:
            ASSERT(FALSE);
            break;
        }
    }
    return bSame;
}

CModalDialogMgr::~CModalDialogMgr()
{

}
void CModalDialogMgr::RegisterMfcModalDlg(CDialog* pDialog)
{
    CSingleLock locker(&m_csLock, TRUE);
    ModalWndInfo mfcModalInfo;
    mfcModalInfo.type = mtMfcDialog;
    mfcModalInfo.pDialog = pDialog;

#ifdef _DEBUG
    ASSERT(_FindModalWnd(mfcModalInfo) == NULL);
#endif 

    m_allModalWnds.push_back(mfcModalInfo);
}

void CModalDialogMgr::UnRegisterMfcModalDlg(CDialog* pDialog)
{
    CSingleLock locker(&m_csLock, TRUE);

    ModalWndInfo mfcModalInfo;
    mfcModalInfo.type = mtMfcDialog;
    mfcModalInfo.pDialog = pDialog;

#ifdef _DEBUG
    ASSERT(m_allModalWnds.empty() || _FindModalWnd(mfcModalInfo) != NULL);
#endif 
    m_allModalWnds.remove(mfcModalInfo);
}


void CModalDialogMgr::RegisterDuiModalWnd(CWindowWnd* pDuiModalWnd)
{
    CSingleLock locker(&m_csLock, TRUE);
    ModalWndInfo duiWndInfo;
    duiWndInfo.type = mtDuiWnd;
    duiWndInfo.pDuiModalWnd = pDuiModalWnd;

#ifdef _DEBUG
    ASSERT(_FindModalWnd(duiWndInfo) == NULL);
#endif 
    m_allModalWnds.push_back(duiWndInfo);
}

void CModalDialogMgr::UnRegisterDuiModalWnd(CWindowWnd* pDuiModalWnd)
{
    CSingleLock locker(&m_csLock, TRUE);
    ModalWndInfo duiWndInfo;
    duiWndInfo.type = mtDuiWnd;
    duiWndInfo.pDuiModalWnd = pDuiModalWnd;

#ifdef _DEBUG
    ASSERT(m_allModalWnds.empty() || _FindModalWnd(duiWndInfo) != NULL);
#endif 
    m_allModalWnds.remove(duiWndInfo);
}

BOOL CModalDialogMgr::CloseAllModalDialog()
{
    BOOL bRet = TRUE;
    CSingleLock locker(&m_csLock, TRUE);

    //close from back 
    for (ModalWndContainer::reverse_iterator iter = m_allModalWnds.rbegin(); 
        iter != m_allModalWnds.rend();
        iter++)
    {
        ModalWndInfo& tmpWndInfo = *iter;
        switch (tmpWndInfo.type)
        {
        case mtMfcDialog:
            {
                ASSERT(tmpWndInfo.pDialog && ::IsWindow(tmpWndInfo.pDialog->m_hWnd));
                if (tmpWndInfo.pDialog)
                {
                    tmpWndInfo.pDialog->EndDialog(IDCANCEL);
                }
                break;
            }
        case mtDuiWnd:
            {
                ASSERT(tmpWndInfo.pDuiModalWnd && ::IsWindow(tmpWndInfo.pDuiModalWnd->GetHWND()));
                if (tmpWndInfo.pDuiModalWnd)
                {
                    tmpWndInfo.pDuiModalWnd->Close(IDCANCEL);
                }
                break;
            }
        default:
            ASSERT(FALSE);
            break;
        }
    }

    m_allModalWnds.clear();

    return bRet;
}


ModalWndInfo* CModalDialogMgr::_FindModalWnd(const ModalWndInfo& wndInfo)
{
    ModalWndInfo* pWndInfo = NULL;

    CSingleLock locker(&m_csLock, TRUE);
    for (ModalWndContainer::reverse_iterator iter = m_allModalWnds.rbegin(); 
        iter != m_allModalWnds.rend();
        iter++)
    {
        BOOL bSame = FALSE;
        ModalWndInfo& tmpWndInfo = *iter;
        if (tmpWndInfo.type == wndInfo.type)
        {
            switch (tmpWndInfo.type)
            {
            case mtMfcDialog:
                bSame = (tmpWndInfo.pDialog == wndInfo.pDialog);
                break;
            case mtDuiWnd:
                bSame = (tmpWndInfo.pDuiModalWnd == wndInfo.pDuiModalWnd);
                break;
            default:
                ASSERT(FALSE);
                break;
            }
            if (bSame)
            {
                pWndInfo = &(*iter);
                break;
            }
        }
    }

    return pWndInfo;
}

INT CModalDialogMgr::GetModalWndCount()
{
    CSingleLock locker(&m_csLock, TRUE);
    return (INT)m_allModalWnds.size();
}